@@ -1,9 +1,12 @@
 #' Set query threshold
 #' 
+#' Depreciated. This function will be remove in the next version.
+#' 
 #' @param worker a jieba worker
 #' @param number the query threshold
 #' @export
 query_threshold = function(worker, number){
+  warning("This function is depreciated, and will be removed in the next version due to the upstream apis changes.")
   stopifnot(inherits(worker,"jieba"))
   set_query_threshold(number,worker$worker)
   worker$max_word_length = number
@@ -13,9 +16,13 @@ query_threshold = function(worker, number){
 #' 
 #' @param worker a jieba worker
 #' @param words the new words
-#' @param tags the new words tags
+#' @param tags the new words tags, default "n"
+#' @examples
+#' cc = worker()
+#' new_user_word(cc, "test")
+#' new_user_word(cc, "do", "v")
 #' @export
-new_user_word = function(worker, words, tags){
+new_user_word = function(worker, words, tags = rep("n", length(words))){
   stopifnot(inherits(worker,"jieba"), length(words) == length(tags), is.character(words), is.character(tags))
   if (.Platform$OS.type == "windows") {
     words <- enc2utf8(words)
@@ -25,10 +32,19 @@ new_user_word = function(worker, words, tags){
 }
 #' Get text location
 #' 
+#' Depreciated. This function will be remove in the next version.
+#' 
 #' @param words a string
 #' @return a list with words, start position, and end position
 #' @export
+#' @examples 
+#' \dontrun{
+#' words_locate(c("this","is","a", "test"))
+#' }
+#' 
+#' 
 words_locate = function(words){
+  warning("This function is depreciated, and will be removed in the next version due to the upstream apis changes.")
   if (.Platform$OS.type == "windows") {
     words <- enc2utf8(words)
   }
